# t/lib/Dist/Zilla/Plugin/ErrorLoggerTestPlugin.pm #

#   This file is part of perl-Dist-Zilla-Role-ErrorLogger.

package Dist::Zilla::Plugin::ErrorLoggerTestPlugin;

use Moose;
use namespace::autoclean;

with 'Dist::Zilla::Role::FileGatherer';
with 'Dist::Zilla::Role::ErrorLogger';

sub gather_files {
    my ( $self ) = @_;
    $main::Test->( $self );
    return;
};

__PACKAGE__->meta->make_immutable;

no Moose;

1;

# end of file #
